import mx.events.EventDispatcher;
import mx.utils.Delegate;

import YouTubePlayerEvent;

class YouTubePlayer
{
	public var autoPlay:Boolean									= true;
	public var chromeless:Boolean								= false;
	public var pars:String										= 'autoplay=0&loop=0&rel=0&showsearch=0&hd=1';
	public var playerWidth:Number								= 425;
	public var playerHeight:Number								= 344;
	
	public var addEventListener:Function;
	
	private var intervals:Array									= [ ];
	
	private var parent:MovieClip;
	private var loader:MovieClipLoader;
	private var player:MovieClip;
	private var videoId:String;
	private var removePlayer:Boolean;
	private var dispatchEvent:Function;
	
	public function YouTubePlayer(parent:MovieClip)
	{
		EventDispatcher.initialize( this );
		
		System.security.allowDomain( '*' );
		System.security.allowDomain( 'www.youtube.com' );
		System.security.allowDomain( 'youtube.com' );
		System.security.allowDomain( 's.ytimg.com' );
		System.security.allowDomain( 'i.ytimg.com' );
		
		this.parent = parent;
		
		loader = new MovieClipLoader();
	}
	
	public function init(videoId:String):Void
	{
		this.videoId = videoId;
		
		if ( player )
		{
			removePlayer = true;
			
			if ( player.getPlayerState() != 1 )
			{
				destroyPlayer();
			}
			else
			{
				player.stopVideo();
			}
		} 
		else 
		{		
			loadVideo(); 
		}
	}
	
	private function loadVideo():Void
	{
		var url:String = ( chromeless ? 'http://www.youtube.com/apiplayer' : 'http://www.youtube.com/v/' + videoId + '&' + pars );
		
		player = parent.createEmptyMovieClip( 'Player' + ( new Date().getTime() ), parent.getNextHighestDepth() );
		
		parent._visible = false;
		
		loader.addListener({ onLoadInit: loadInit() });
		
		loader.unloadClip( player );
		
		loader.loadClip( url, player );
	}
	
	private function loadInit():Void
	{
		intervals.push( setInterval( Delegate.create( this, checkPlayerLoaded ) , 500 ) );
	}
	
	private function checkPlayerLoaded():Void
	{
		if ( player.isPlayerLoaded() )
		{			
			if ( chromeless )
			{
				player.loadVideoById( videoId );
			}
			
			player.addEventListener( YouTubePlayerEvent.STATE_CHANGED, Delegate.create( this, onPlayerStateChanged ) ); 
			player.addEventListener( YouTubePlayerEvent.ERROR, Delegate.create( this, onPlayerError ) );
			
			resizePlayer( playerWidth, playerHeight );
			
			parent._visible = true;
			
			for ( var i:Number = 0; i < intervals.length; i++ )
			{
				clearInterval( intervals[ i ] );
			}
			
			onPlayerReady();
		}
	}
	
	private function onPlayerStateChanged(state:Number):Void
	{
		switch ( state )
		{
			case 0:
			// ended
			
			if ( removePlayer )
			{
				destroyPlayer();
			}
			
			dispatchEvent( new YouTubePlayerEvent( YouTubePlayerEvent.PLAYER_ENDED ) );
			
			break;
			
			case 1:
			// playing
			
			dispatchEvent( new YouTubePlayerEvent( YouTubePlayerEvent.PLAYER_PLAYING ) );
			
			break;
			
			case 2:
			// paused
			
			dispatchEvent( new YouTubePlayerEvent( YouTubePlayerEvent.PLAYER_PAUSED ) );
			
			break;
			
			case 3:
			// buffering
			
			dispatchEvent( new YouTubePlayerEvent( YouTubePlayerEvent.PLAYER_BUFFERING ) );
			
			break;
			
			case 5:
			// video queued
			
			dispatchEvent( new YouTubePlayerEvent( YouTubePlayerEvent.PLAYER_QUEUED ) );
			
			break;
		}
	}
	
	private function onPlayerError():Void
	{
		dispatchEvent( new YouTubePlayerEvent( YouTubePlayerEvent.ERROR ) );
	}
	
	public function resizePlayer(width:Number, height:Number):Void
	{
		if ( player.isPlayerLoaded() )
		{
			player.setSize( width, height );
		}
	}
	
	private function onPlayerReady():Void
	{
		if ( autoPlay )
		{
			player.playVideo();
		}
	}
	
	private function destroyPlayer():Void
	{	
		intervals.push( setInterval( Delegate.create( this, checkPlayerDestroyed ), 500 ) );
		
		removePlayer = false;
	}
	
	private function checkPlayerDestroyed():Void
	{
		if ( !player )
		{
			for ( var i:Number = 0; i < intervals.length; i++ )
			{
				clearInterval( intervals[ i ] );
			}
			
			init( videoId );
		}
		else
		{
			for ( var players:String in parent )
			{
				parent[ players ].destroy();

				removeMovieClip( parent[ players ] );
				unloadMovie( parent[ players ] );
			}
			
			player = null;
		}
	}
	
	public function stop():Void
	{
		removePlayer = false;
		
		player.stopVideo();
	}
}